c       Th. V. Karambas and C. Koutitas
c       Hyperbolic type 'mild slope' equations
c       based on Yu-Isobe-Watanabe paper (CEJ, 1992)
c       refraction-diffraction-reflection inside harbours
	 
								    
c	implicit real*8 (a-h,o-z)	  	    	    
	 						 	   	  		 
        parameter (ijm=1700,iim=900)	  	   
	 								  	   
c       u and v are the velocities		      
c       z is the elevation				     
c       d is the depth					   
c       c is the linear theory celerity   

       dimension u(ijm,iim),v(ijm,iim),vn(ijm,iim)
	dimension uc(ijm,iim),vc(ijm,iim), thitn(ijm,iim)
       dimension dnmax(ijm,iim), H(ijm,iim)
       dimension z(ijm,iim),c(ijm,iim),un(ijm,iim)
       dimension d(ijm,iim),r(ijm), icod(ijm,iim)
       dimension ed(ijm,iim),edr(ijm,iim),frrl(ijm,iim)
       dimension umax(ijm,iim),vmax(ijm,iim)
       dimension edrl(ijm,iim),fr(ijm,iim)
	dimension sumx1(ijm,iim), sumy1(ijm,iim)
	dimension sum2(ijm,iim), sum3(ijm,iim),sum4(ijm,iim)
	dimension sumz(ijm,iim), sumxy(ijm,iim), an(ijm,iim)
	dimension qbm(ijm,iim), qbmax(ijm,iim),ze(ijm,iim)
	dimension Hrms(ijm,ijm), sumu(ijm,iim), sumv(ijm,iim)
	dimension sigu(ijm,ijm), sigv(ijm,iim), thita(ijm,iim)
		dimension amflx(ijm,iim), amfly(ijm,iim)

c       Ho:     wave height
c       dt, dx: time and space step
c       per:    the period
c       d       water depth
c       im, jm: max number of grids (x and y)
c       tend:   time in which the programme stops

c       c is the celerity
c       dnmax the max elevation	  
c       at each point

	write (*,*)'  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~'
	write (*,*)'  ~ Th. V. Karambas and C. Koutitas          ~'
	write (*,*)'  ~ Hyperbolic type "mild slope"  equations  ~'
	write (*,*)'  ~ Refraction-diffraction-reflection        ~'
	write (*,*)'  ~ of linear monochromatic waves inside     ~'
	write (*,*)'  ~ port and harbours                        ~'
	write (*,*)'  ~                                          ~'  
	write (*,*)'  ~ Ho:     wave height                      ~'
	write (*,*)'  ~ dt, dx: time and space step              ~'
	write (*,*)'  ~ T:    the period                         ~'
	write (*,*)'  ~ im, jm: max number of grids (x and y)    ~'
	write (*,*)'  ~ tend: time in which the programme stops  ~'
	write (*,*)'  ~ angle: the incident wave angle (in rad)  ~'
	write (*,*)'  ~                                          ~'  
	write (*,*)'  ~ Input file:                              ~'  
	write (*,*)'  ~ "depth.dat"  (depth and boundary file)   ~'
	write (*,*)'  ~                                          ~'
	write (*,*)'  ~ Output files:                            ~'  
	write (*,*)'  ~ "height.dat" (wave height)               ~'  
	write (*,*)'  ~ "section.dat" (wave height in sections   ~'  
	write (*,*)'  ~  x=im/4, x=im/2 and x=3im/4)             ~'  
	write (*,*)'  ~ "elev.dat" (surface elevation)           ~'  
	write (*,*)'  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~'


	data Ho,per,dt,dx,fin,tend/1.5,7.0,.0125,2.5,0.0,400./
	data slope/50.0/
	aLo=9.81*per**2/2./3.14159
	gamma=.5+sqrt(Ho/aLo)
c   if periodic gamma=.85+sqrt(Ho/aLo)
c	gamma=1.5+sqrt(Ho/aLo)
	gamma=1.+sqrt(Ho/aLo)

		
	tna=1./slope

	a1=(2.2+2.*tanh(55*tna))*tanh((1.6*tna**-1.5+25)*Ho/aLo)
     &  +30*tna**.1*Ho/aLo
	a2=9.5*(Ho/aLo)**-.5+10


	dhr=-.25
	dhs=.08

	dhr=-.1*Ho
	dhs=.25*Ho

		 
	pi=3.141593		 	   		 
	 					 	  		  	   
	omega=2.*pi/per	
	sigma=omega		 			    


c       read the depth

	open (unit=20,file='depth.dat')

	write (*,*) 'reading depth.dat file'

        
	read (20,*) im,jm
	do j=1,jm
	read (20,*) ja					    
	write (*,*) ja
	read (20,*) (d(i,j),i=1,im)
	end do

c	OPEN(13,FILE='vel.dat')


c	do j=21,JM
c	do i=1,IM

c	read (13,*) ai,aj,Uc(i,j),Vc(i,j),ze(i,j)

c	end do
c	end do
c	close (13)   

c	 open (unit=3,file='thita.dat')
c		do j=1,jm
c	read (3,*) ja
c		do i=1,im
c	read (3,*) ia,thitn(i,j),aa,bb
c		end do
c		end do
c		close (3)   





	do j=1,jm
	do i=1,im
	edrl(i,j)=0.
	Uc(i,j)=0.
	Vc(i,j)=0.
	ze(i,j)=0.
	end do
	end do
 
	do j=1,jm
	do i=1,im
c	edr(i,j)=edrl(i,j)
c	edr(i,j)=0.
c	frrl(i,j)=edrl(i,j)
	end do
	end do

c open(unit=18,file='see.dat')
c do i=1,im,3						    
c do j=1,jm,3
c write (18,6)i*dx,j*dx,d(i,j)
c end do
c end do
c close (18)
c stop

	OPEN(13,FILE='icod.dat')

      DO 200 J=1,JM
      read (13,*) ija

      READ(13,*)(icod(I,J),I=1,IM)	   

200    continue
      CLOSE(13)

	     	
	do i=1,im
	do j=1,jm
	d(i,j)=d(i,j)
c	+ze(i,j)
	if (d(i,j).gt.dhr.and.d(i,j).lt.dhs) d(i,j)=dhs
	if (d(i,j).le.dhr) icod(i,j)=1
	end do
	end do



c        do j=1,jm
c        write (*,*) d(im/2,j)
c        end do


	write (*,*) 
	write (*,*) 
	write (*,*)'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~'
	write (*,*)'~     the program is now running !!!       ~'
	write (*,*)'~                                          ~'         
	write (*,*)'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~'
	write (*,*)'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~'

	write (*,*) '   Time in secs'

	do 30 i=1,im
	do 30 j=1,jm
	u(i,j)=0.0
	v(i,j)=0.0
	z(i,j)=0.0
	un(i,j)=0.0
	vn(i,j)=0.0
	umax(i,j)=0.
	vmax(i,j)=0.
	dnmax(i,j)=0.0
	qbmax(i,j) =0.
	qbm(i,j)=0.
30      continue


c       lo=aleno
	aleno=9.81*(per**2)/2.0/3.14


c       solving the linear dispersion relation
c       to estimate celerity c

	do 876 j=1,jm
	do 876 i=1,im

	if (icod(i,j).eq.1) goto 876
	
	alen=aleno
39      alenp=alen
	akapa=2*3.14/alenp
	akd=akapa*d(i, j)
	alen = aleno *tanh(akd)
	dlen = abs(alen - alenp)
	if (dlen.gt.0.001) goto 39

	c(i,j)=9.81*per/(2*3.14)*tanh(akd)
	an(i,j)=0.5*(1+2*akd/sinh(2*akd))

876     continue

	

      jst=30
	jfin=2*jst
	delta=.3
	aln=c(im/2,jst)*per
      wa=delta*aln/2.
      bta=80./((delta*aln)**2)


	do j=1,jm-1
	do i=1,im-1

	if (icod(i,j+1).eq.1) then
	c(i,j+1)=c(i,j)
	an(i,j+1)=an(i,j)
	else
	end if

	if (icod(i+1,j).eq.1) then
	c(i+1,j)=c(i,j)
	an(i+1,j)=an(i,j)
	else
	end if

	end do
	end do



	t=0.0
	n=0
40    t=t+dt
	n=n+1

c
c       continuity equation
c
	do 100 j=1,jm-1
	do 100 i=1,im-1

	if (icod(i,j).eq.1) then
	z(i,j)=0.0 
	goto 100
	else
	end if


	q1=.5*(d(i,j)+d(i+1,j))*u(i+1,j)
	q2=.5*(d(i,j)+d(i-1,j))*u(i,j)
	ux=(q1-q2)/dx

	p1=.5*(d(i,j)+d(i,j+1))*v(i,j+1)
	p2=.5*(d(i,j)+d(i,j-1))*v(i,j)
	vy=(p1-p2)/dx


	z(i,j)=z(i,j)-dt*(ux+vy)

100     continue


c
c       momentum equations (x and y)
c
	do 120 j=2,jm-1
	
	do 120 i=2,im-1

	if (icod(i,j).eq.1.or.icod(i-1,j).eq.1) then
	un(i,j)=0.0 
	goto 120
	else
		
	end if

	
	zcx=(an(i,j)*z(i,j)-an(i-1,j)*z(i-1,j))/dx
		cm=.5*(c(i,j)+c(i-1,j))
	anm=.5*(an(i,j)+an(i-1,j))
	dm=.5*(d(i,j)+d(i-1,j))
	edm=.5*(ed(i,j)+ed(i-1,j))

	uxx=(u(i+1,j)-2*u(i,j)+u(i-1,j))/dx**2
	uyy=(u(i,j+1)-2*u(i,j)+u(i,j-1))/dx**2

	un(i,j)=u(i,j)-dt*cm**2*zcx/dm/anm+
     !  edm*dt*uxx+ed(i,j)*dt*uyy-dt*fr(i,j)*omega*u(i,j)

120     continue

	do 121 j=2,jm-1
	do 121 i=2,im-1


	if (icod(i,j).eq.1.or.icod(i,j-1).eq.1) then
	vn(i,j)=0.0 
	goto 121
	else
	end if

	zcy=(an(i,j)*z(i,j)-an(i,j-1)*z(i,j-1))/dx
	cm=.5*(c(i,j)+c(i,j-1))
	anm=.5*(an(i,j)+an(i,j-1))
	dm=.5*(d(i,j)+d(i,j-1))
	edm=.5*(ed(i,j)+ed(i,j-1))

	vxx=(v(i+1,j)-2*v(i,j)+v(i-1,j))/dx**2
	vyy=(v(i,j+1)-2*v(i,j)+v(i,j-1))/dx**2

	  
	vn(i,j)=v(i,j)-dt*cm**2*zcy/dm/anm+ 
     !  edm*dt*(vxx+vyy)-dt*fr(i,j)*omega*v(i,j)

121     continue

c       do i=1,im
c       do j=1,jm
c       if (d(i,j).gt..01) then
c       uv=(umax(i,j)**2+vmax(i,j)**2)
c       uvl=(.7*c(i,j))**2
c       uv=H(i,j)
c       uvl=d(i,j)
c       Qb=1.-exp(-(uv/uvl)**10)
c       if (Qb.gt.1) Qb=1.
c       dis=Qb*2.*(9.81*(H(i,j)**2)/8.)/per
c       ed(i,j)=edr(i,j)+7.*d(i,j)*dis**.333333
c       ed(i,j)=7.*d(i,j)*dis**.333333
c       else
c       end if
c       end do
c       end do



c       input of the wave surface elevation

	do 81 i=25,im-25
c	jv1=int(tan(fin)*(i-20))+20
	jv1=26
c       write (*,*) i,jv1
	cel=c(i,jv1)
	tf=tan(fin)*i*dx/cel				 
c       tf=0
	if (icod(i,jv1).eq.1) then
		zin=0.
	else
	zin=Ho/2.*sin(2*pi*(t-tf)/per)*cel*dt/dx*cos(fin)
	end if
	
	z(i,jv1)=z(i,jv1)+2.0*zin*tanh(t/per/10.)


81      continue
c	do 82 i=2,im-1
c	jv2=int(tan(fin)*(i-20))+21
c	jv2=31
c	cel=c(i,jv2)
c	tf=tan(fin)*i*dx/cel
c       tf=0
c	zin=Ho/2.*sin(2*pi*(t-tf)/per)*cel*dt/dx*cos(fin)
c82      z(i,jv2)=z(i,jv2)+zin

       write (*,*) t




		do i=1,IM
		if (icod(i,j).eq.0) then
	VN(i,1)=(1.-r(i))*V(i,1)+r(i)*V(i,2)
	Z(i,1)=-d(i,1)*VN(I,1)/c(i,1)
	else
	VN(i,1)=0
	 Z(i,1)=0
	end if
	
	end do
	do i=1,IM
	UN(i,1)=UN(i,2)
	end do
	do i=1,IM
	dven=V(i,3)-V(i,2)
	if (ABS(dven).lt..0001) goto 207
	r(i)=(VN(i,2)-V(i,2))/dven
	if (r(i).gt..99) r(i)=.99
	if (r(i).lt.0.) goto 207
	goto 60
  207   r(i)=0.0
   60   end do


	do j=1,jm
	un(1,j)=u(2,j)
	vn(1,j)=vn(2,j)
	end do


c       boundary condition  sponge layer

c        West b. c.
350     do 165 j=1,jm
       do 165 i=25,2,-1
       ml1=i-25
       ml2=-25.0
       rm=exp((1.06**(ml1)-1.06**(ml2))*2.)
       z(25-i,j)=z(25-i,j)/rm
       vn(25-i,j)=vn(25-i,j)/rm
165     un(25-i,j)=un(25-i,j)/rm
       ims1=im-10
c       do 175 j=1,jm
c       do 175 i=40,20,-1
c       ml1=20-i
c       ml2=-20.0
c       rm=exp((1.8**(ml1)-1.8**(ml2))*1.6)
cc	write(*,*) i,rm
c       z(i,j)=z(i,j)/rm
c       vn(i,j)=vn(i,j)/rm
c175     un(i,j)=un(i,j)/rm

c        South b.c.
	do 665 i=1,im
	do 665 j=1,25
	ml1=j-25
	ml2=-25.0
	rm=exp((1.06**(ml1)-1.06**(ml2))*2.)	
	z(i,25-j)=z(i,25-j)/rm
	vn(i,25-j)=vn(i,25-j)/rm
665     un(i,25-j)=un(i,25-j)/rm
c	do 675 i=1,im
c	do 675 j=20,10,-1
c	ml1=10-j
c	ml2=-10
c	rm=exp((1.8**(ml1)-1.8**(ml2))*1.6)
c	z(i,j)=z(i,j)/rm
c	vn(i,j)=vn(i,j)/rm
c675     un(i,j)=un(i,j)/rm

c        North b.c.
       do 667 i=1,im
       do 667 j=jm-25,jm
       ml1=j-jm
       ml2=-25.0
       rm=exp((1.06**(ml1)-1.06**(ml2))*2.)
       z(i,j)=z(i,j)/rm
       vn(i,j)=vn(i,j)/rm
667     un(i,j)=un(i,j)/rm
c       jms1=jm-10
c       do 677 i=1,im
c       do 677 j=jm,jms1,-1
c       ml1=j-jm
c       ml2=-10
c       rm=exp((1.8**(ml1)-1.8**(ml2))*1.6)
c       z(i,j)=z(i,j)/rm
c       vn(i,j)=vn(i,j)/rm
c677     un(i,j)=un(i,j)/rm

c        East b.c.
       do 668 j=1,jm
       do 668 i=im-25,im
       ml1=i-im
       ml2=-25.0
       rm=exp((1.06**(ml1)-1.06**(ml2))*2.)
	z(i,j)=z(i,j)/rm
       vn(i,j)=vn(i,j)/rm
668     un(i,j)=un(i,j)/rm
       ims1=im-10
c       do 679 j=1,jm
c       do 679 i=im,ims1,-1
c       ml1=i-im
c       ml2=-10
c       rm=exp((1.8**(ml1)-1.8**(ml2))*1.6)
c       z(i,j)=z(i,j)/rm
c       vn(i,j)=vn(i,j)/rm
c679     un(i,j)=un(i,j)/rm

c	write (*,*) t

	do 405 i=1,im
	do 405 j=1,jm
	u(i,j)=un(i,j)
405     v(i,j)=vn(i,j)

	
	  do i=1,im			  
	  do j=1,jm


	  if (icod(i,j).eq.0) then
	  uv=(umax(i,j)**2+vmax(i,j)**2)
	  uvl=(.7*c(i,j))**2
	  uv=(2.*dnmax(i,j))**2
	  uvl=(gamma*d(i,j))**2
	  Qb=1.-exp(-(uv/uvl)**10)
c	if (Qb.lt..001) Qb=0.
	  if (Qb.gt.1.) Qb=1.


	qbm(i,j)=Qb

	He=2.*dnmax(i,j)
	if (d(i,j).lt.dhs) He=dhs
	

	  dis=Qb*2.*(9.81*(He**2)/8.)/per

	hd=d(i,j)
	sb=abs(d(i,j+1)-d(i,j-1))/2./dx
	alf=2.*.333*sb*per*sqrt(9.81/hd)
	if (alf.lt.1.) alf=1.
		alf=1.
	dis=dis*alf


c	if (j.le.72) dis=0.

c	aL=c(i,j)*per
c	Hmax=.14*aL*tanh(.91*2.*pi*d(i,j)/aL)
c	Hmax=.8*d(i,j)
c	HeHm=He/Hmax
c	Qb=1.785*HeHm**3-.28*HeHm**2-.738*HeHm+.235
c	if (HeHm.le..43) Qb=0.
c	if (HeHm.ge.1.) Qb=1.
c	dis=Qb*9.81*Hmax**2/4./per

c	Hmax=.55*d(i,j)
c	dis=.4*9.81*(He**2-Hmax**2)/per
c	if (dis.lt.0) dis=0.
c	if (d(i,j).le.dhs) dis=2.*(9.81*(dhs**2)/8.)/per
c       ed(i,j)=edr(i,j)+7.*d(i,j)*dis**.333333

	  ed(i,j)=12.*He*dis**.333333

c	if (d(i,j).le.dhs) ed(i,j)=.5*dhs*sqrt(9.81*dhs)
c	  if (d(i,j).le.1.1*dhs) ed(i,j)=.3*d(i,j)*sqrt(9.81*d(i,j))
c	ed(i,j)=0.
c	fr(i,j)=frrl(i,j)+2.5/slope*sqrt(9.81/d(i,j))*Qb
	fr(i,j)=frrl(i,j)+2.5/slope*sqrt(9.81/d(i,j))*Qb
	fr(i,j)=0.
c	  if (edr(i,j).gt.ed(i,j)) ed(i,j)=edr(i,j)
	  if (ed(i,j).gt.18.) ed(i,j)=18.
	  else
	  end if
	  end do
	  end do


	do j=2,jm-1
	do i=2,im-1
	ed(i,j)=
     &.25*ed(i-1,j)+.25*ed(i+1,j)+.25*ed(i,j-1)+.25*ed(i,j+1)
	end do
	end do

	do j=2,jm-1
	do i=2,im-1
	ed(i,j)=
     &.25*ed(i-1,j)+.25*ed(i+1,j)+.25*ed(i,j-1)+.25*ed(i,j+1)
	end do
	end do


	do 185 j=1,jm
	do 185 i=1,im

	aL=c(i,j)*per
	coef=1.+a1*exp(-a2*d(i,j)/aL)
	coef=1.		

	if (icod(i,j).eq.1) goto 185
	   if (coef*z(i,j).gt.dnmax(i,j)) then
	   dnmax(i,j) = coef*z(i,j)   
	   else
	   end if
		
	   if (u(i,j).gt.umax(i,j)) then
	   umax(i,j) = u(i,j)
	   else
	   end if
	   if (v(i,j).gt.vmax(i,j)) then
	   vmax(i,j) = v(i,j)
	   else
	   end if
185        continue




	nper=int(3.*per/dt)
	nendh=int(tend/dt-nper)

	if (n.ge.nendh) then
	do 110 i=2,im-1
	do 110 j=2,jm-1

	rx=.5*(u(i,j)+u(i+1,j))*(d(i,j)+z(i,j))
	ry=.5*(v(i,j)+v(i,j+1))*(d(i,j)+z(i,j))

	rx1=u(i+1,j)*.5*(d(i+1,j)+d(i,j)+z(i+1,j)+z(i,j))
	rx2=u(i,j)*.5*(d(i-1,j)+d(i,j)+z(i-1,j)+z(i,j))
	ry1=v(i,j+1)*.5*(d(i,j+1)+d(i,j)+z(i,j+1)+z(i,j))
	ry2=v(i,j)*.5*(d(i,j-1)+d(i,j)+z(i,j-1)+z(i,j))

	drx=(rx1-rx2)/dx
	dry=(ry1-ry2)/dx

	sumx1(i,j)=sumx1(i,j)+rx**2
	sumy1(i,j)=sumy1(i,j)+ry**2

	sum2(i,j)=sum2(i,j)+(drx+dry)**2

	sum3(i,j)=sum3(i,j)+rx*(drx+dry)
	sum4(i,j)=sum4(i,j)+ry*(drx+dry)

	sumz(i,j)=sumz(i,j)+z(i,j)**2

	sumxy(i,j)=sumxy(i,j)+rx*ry
	all=c(i,j)*per
	akd=2*pi*d(i,j)/all
	ub=akd*u(i,j)/sinh(akd)
	vb=akd*v(i,j)/sinh(akd)
	sumu(i,j)=sumu(i,j)+ub**2
	sumv(i,j)=sumv(i,j)+vb**2

	amflx(i,j)=amflx(i,j)+z(i,j)*u(i,j)*9.81*akd/c(i,j)/sigma**2
	amfly(i,j)=amfly(i,j)+z(i,j)*v(i,j)*9.81*akd/c(i,j)/sigma**2

110      continue


	else
	end if


c
c       equalies the grids outside the calculated region
c       with z=2.*Ho in order to have a nice view in the
c       surfer package 

	nend=int(tend/dt)-1
	if (n.lt.nend) then
	goto 40
	      else						 
		do 321 j=1,jm
		do 321 i=1,im
		if (icod(i,j).eq.1) then
		z(i,j)=Ho
		else
		end if
321             continue



	open (unit=3,file='elev.dat')
	open (unit=5,file='hei.dat')
	open (unit=10,file='sxxyy.dat')
	open (unit=19,file='bottom.dat')
	open (unit=18,file='thita.dat')
	open (unit=17,file='sect.dat')
	open (unit=21,file='uxut.dat')
	open (unit=22,file='untow.dat')



		do 355 j=20+1,jm-3
		do 355 i=4,im-3 
		disx=i*dx
		disy=j*dx
		write (3,7) disx,disy,z(i,j),qbmax(i,j)
355             continue




777     FORMAT (6F12.7)
797     FORMAT (20i3)


	do 114 j=2,jm-1
	do 114 i=2,im-1

	if (icod(i,j).eq.0) then
	al=c(i,j)*per
	akapa=2*pi/al
	akd=d(i,j)*akapa

	ro=1000

	alfa=ro*akapa*(sinh(2*akd)+2*akd)/4/(sinh(akd))**2
	bhta=ro*(sinh(2*akd)-2*akd)/4/akapa/(sinh(akd))**2
	di=ro*d(i,j)*((sinh(2*akd))/2/akd-cosh(2*akd))/4/(sinh(akd))**2

 
	trx1=alfa*sumx1(i,j)
	trx2=bhta*sum2(i,j)
	trx3=di*(sum3(i+1,j)-sum3(i-1,j))/2/dx
	trx4=di*(sum4(i,j+1)-sum4(i,j-1))/2/dx
	trx5=0.5*ro*9.81*sumz(i,j)
c       u=sxx
	u(i,j)=(trx1-trx2+trx3+trx4+trx5)/nper

	try1=alfa*sumy1(i,j)
	try2=bhta*sum2(i,j)
	try3=di*(sum3(i+1,j)-sum3(i-1,j))/2/dx
	try4=di*(sum4(i,j+1)-sum4(i,j-1))/2/dx
	try5=0.5*ro*9.81*sumz(i,j)
c       v=syy
	v(i,j)=(try1-try2+try3+try4+try5)/nper

c       un=sxy
	un(i,j)=alfa*sumxy(i,j)/nper

	Hrms(i,j)=2.*sqrt(2.)*sqrt(sumz(i,j)/nper)
	sigu(i,j)=2.*sqrt(2.)*sqrt(sumu(i,j)/nper)
	sigv(i,j)=2.*sqrt(2.)*sqrt(sumv(i,j)/nper)

		amflx(i,j)=amflx(i,j)/nper
	amfly(i,j)=amfly(i,j)/nper

	Uh2=abs(sumx1(i,j)/nper)
	Vh2=abs(sumy1(i,j)/nper)
	zc2=abs(sumz(i,j)*(c(i,j)**2)/nper)
	
	if (Vh2.gt.0.0001) then
	thita(i,j)=atan((Uh2/Vh2)**.5)
c	else if (zc2.gt..0001) then
c	thita(i,j)=asin((Uh2/zc2)**.5)
	else if (Uh2.gt.0.0001) then
	thita(i,j)=0.
	else
	thita(i,j)=1.570796
	end if

	else
	u(i,j)=0.0
	v(i,j)=0.0
	un(i,j)=0.0
	end if

114      continue


	do i=1,im
	u(i,1)=u(i,2)
	v(i,1)=v(i,2)
	un(i,1)=un(i,2)
	u(i,jm)=u(i,jm-1)
	v(i,jm)=v(i,jm-1)
	un(i,jm)=un(i,jm-1)
	end do
	do j=1,jm
	u(1,j)=u(2,j)
	v(1,j)=v(2,j)
	un(1,j)=un(2,j)
	u(im,j)=u(im-1,j)
	v(im,j)=v(im-1,j)
	un(im,j)=un(im-1,j)
	end do



		do j=1,jm
	write (18,17) j
		do i=1,im

	write (18,17) i,thita(i,j),2*dnmax(i,j),c(i,j)

		disx=i*dx
		disy=j*dx
	if (icod(i,j).eq.0.and.vmax(i,j).gt..01) then
	th=atan(umax(i,j)/vmax(i,j))
	uvmax=sqrt(umax(i,j)**2+vmax(i,j)**2)
	ths=asin(vmax(i,j)/uvmax)
	else
	th=0.0
	ths=0.0
	end if
	
       if (icod(i,j).eq.0) then
c       uvmax=sqrt(umax(i,j)**2+vmax(i,j)**2)

       write (5,6) disx,disy,2.*dnmax(i,j)
       else
	hh=2.*Ho
	       write (5,6) disx,disy,hh
       end if
		end do
		end do

	do i=21,im-20,10
	write(17,*) i*dx
	do j=1,jm

	if (icod(i,j).eq.0.and.vmax(i,j).gt..01) then
	th=atan(umax(i,j)/vmax(i,j))
	uvmax=sqrt(umax(i,j)**2+vmax(i,j)**2)
	ths=asin(vmax(i,j)/uvmax)
        write (17,6) j*dx,2*dnmax(i,j),d(i,j),th
	else
	th=0.0
	ths=0.0
	end if	
	end do
	end do

	do j=1,jm
	do i=1,im
	dtr=d(i,j)-dnmax(i,j)
	write (22,*) i*dx,j*dx,-amflx(i,j)/dtr,-amfly(i,j)/dtr

	end do
	end do



	open (unit=11,file='sect_H.dat')
	do j=21,jm

	write (11,6) (126-j)*dx,2*dnmax(im/2,j)*1.41,Hrms(im/2,j)*1.41
     &	, thita(im/2,j)
	end do


		do j=1,jm
		do i=1,im

	
	write (19,6) sigu(i,j),sigv(i,j),ed(i,j),qbm(i,j)

	if (i.ge.2.and.j.ge.2.and.i.le.im-1.and.j.le.jm-1) then
	sxx=.5*u(i,j)+.125*u(i-1,j)+.125*u(i+1,j)+
     &  .125*u(i,j-1)+.125*u(i,j+1)
	syy=.5*v(i,j)+.125*v(i-1,j)+.125*v(i+1,j)+
     &  .125*v(i,j-1)+.125*v(i,j+1)
	sxy=.5*un(i,j)+.125*un(i-1,j)+.125*un(i+1,j)+
     &  .125*un(i,j-1)+.125*un(i,j+1)

	else
	sxx=u(i,j)
	syy=v(i,j)
	sxy=un(i,j)
	end if
       write (10,7) sxx,syy,sxy
		end do
		end do



		stop
	      endif
6       format (4f15.6)
7       format (4f15.6)   
17      format (i6,3f12.4)   

	end
	
